/*
 * Copyright 2017-2018 NXP
 * All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NXP "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NXP OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
/*!
 * @file ic_pal.c
 *
 * @page misra_violations MISRA-C:2012 violations
 *
 * @section [global]
 * Violates MISRA 2012 Required Rule 1.3, Taking address of near auto variable.
 * The code is not dynamically linked. An absolute stack address is obtained
 * when taking the address of the near auto variable. A source of error in
 * writing dynamic code is that the stack segment may be different from the data
 * segment.
 *
 * @section [global]
 * Violates MISRA 2012 Advisory Rule 8.7, External could be made static.
 * Function is defined for usage by application code.
 *
 * @section [global]
 * Violates MISRA 2012 Advisory Rule 8.9, An object should be defined at block
 * scope if its identifier only appears in a single function.
 * An object with static storage duration declared at block scope cannot be
 * accessed directly from outside the block.
 *
 * @section [global]
 * Violates MISRA 2012 Required Rule 10.3, Expression assigned to a narrower or different essential type
 * This is needed for the conversion between generic IC types to driver types.
 *
 * @section [global]
 * Violates MISRA 2012 Advisory Rule 10.5, Impermissible cast; cannot cast from 'essentially enum<i>'
 * to 'essentially enum<i>'
 * This is needed for the conversion between generic IC types to the driver types.
 *
 * @section [global]
 * Violates MISRA 2012 Required Rule 10.8, Impermissible cast of composite expression
 *(different essential type categories).
 * This is required by the conversion of unsigned long integer into unsigned integer.
 *
 * @section [global]
 * Violates MISRA 2012 Required Rule 11.3, Cast performed between a pointer to object type
 * and a pointer to a different object type.
 * This is needed for the extension of the user configuration structure, for which the actual type
 * cannot be known.
 *
 * @section [global]
 * Violates MISRA 2012 Advisory Rule 11.5, Conversion from pointer to void to pointer to other type
 * This is needed for the extension of the user configuration structure, for which the actual type
 * cannot be known.
 *
 * @section [global]
 * Violates MISRA 2012 Advisory Rule 11.4, Conversion between a pointer and
 * integer type.
 * The cast is required to initialize a pointer with an unsigned long define,
 * representing an address.
 *
 * @section [global]
 * Violates MISRA 2012 Required Rule 11.6, Cast from unsigned int to pointer.
 * The cast is required to initialize a pointer with an unsigned long define,
 * representing an address.
 *
 */

#include "ic_pal.h"
#include "interrupt_manager.h"
#include "ic_irq.h"

/*******************************************************************************
 * Variables
 ******************************************************************************/

/* Define state structures for IC PAL */
static ic_pal_state_t g_icPalStatePtr[NUMBER_OF_IC_PAL_INSTANCES];

/* IC state-instance matching */
static uint32_t icStateInstanceMapping[NUMBER_OF_IC_PAL_INSTANCES];
/* IC available resources array */
static bool icStateIsAllocated[NUMBER_OF_IC_PAL_INSTANCES];

#if (defined(IC_PAL_OVER_FTM))
/* The FTM state structures */
static ftm_state_t g_ftmState[NUMBER_OF_IC_PAL_INSTANCES];
#endif

/*******************************************************************************
 * Code
 ******************************************************************************/
/*FUNCTION**********************************************************************
 *
 * Function Name : icAllocateState
 * Description   : Allocates one of the available state structure.
 *
 *END**************************************************************************/
static uint8_t icAllocateState(bool * isAllocated,
                               uint32_t * instanceMapping,
                               uint32_t instance)
{
    uint8_t i;

    /* Allocate one of the IC state structure for this instance */
    for (i = 0U;i < NUMBER_OF_IC_PAL_INSTANCES;i++)
    {
        if (isAllocated[i] == false)
        {
            instanceMapping[i] = instance;
            isAllocated[i] = true;
            break;
        }
    }

    return i;
}

/*FUNCTION**********************************************************************
 *
 * Function Name : icFreeState
 * Description   : Deallocates one of the available state structure.
 *
 *END**************************************************************************/
static void icFreeState(bool * isAllocated,
                        const uint32_t * instanceMapping,
                        uint32_t instance)
{
    uint8_t i;

    /* Allocate one of the IC state structure for this instance */
    for (i = 0U;i < NUMBER_OF_IC_PAL_INSTANCES;i++)
    {
        if (instanceMapping[i] == instance)
        {
            isAllocated[i] = false;
            break;
        }
    }
}

/*FUNCTION**********************************************************************
 *
 * Function Name : FindIcState
 * Description   : Search the state structure of the instance for each IP
 *
 *END**************************************************************************/
static uint8_t FindIcState(uint32_t instance)
{
    uint8_t i;

    for (i = 0U;i < NUMBER_OF_IC_PAL_INSTANCES;i++)
    {
        if (icStateInstanceMapping[i] == instance)
        {
            break;
        }
    }

    return i;
}

#if (defined(IC_PAL_OVER_FTM))
/*FUNCTION**********************************************************************
 *
 * Function Name : IC_FtmInit
 * Description   : This function initializes IC_PAL over FTM.
 *
 *END**************************************************************************/
static status_t IC_FtmInit(uint32_t instance,
                           const ic_config_t * configPtr)
{
    ic_pal_state_t * icState;
    status_t status = STATUS_SUCCESS;
    uint8_t index = 0U;

    /* Allocate one of the IC state structure for this instance */
    index = FindIcState(instance);
    icState = &g_icPalStatePtr[index];

    uint8_t channel = 0U;
    ftm_pwm_sync_t sync;
    ftm_user_config_t info;
    ftm_state_t * ftmState = &g_ftmState[index];
    ftm_input_param_t param;
    ftm_input_ch_param_t channelConfig[FEATURE_FTM_CHANNEL_COUNT];

    /* Configure the synchronous to default */
    sync.softwareSync = true;
    sync.hardwareSync0 = false;
    sync.hardwareSync1 = false;
    sync.hardwareSync2 = false;
    sync.maxLoadingPoint = true;
    sync.minLoadingPoint = false;
    sync.inverterSync = FTM_SYSTEM_CLOCK;
    sync.outRegSync = FTM_SYSTEM_CLOCK;
    sync.maskRegSync = FTM_SYSTEM_CLOCK;
    sync.initCounterSync = FTM_SYSTEM_CLOCK;
    sync.autoClearTrigger = false;
    sync.syncPoint = FTM_UPDATE_NOW;

    /* Get some information from user configuration */
    info.syncMethod = sync;
    info.ftmMode = FTM_MODE_INPUT_CAPTURE;
    info.ftmPrescaler = ((extension_ftm_for_ic_t *)(configPtr->extension))->ftmPrescaler;
    info.ftmClockSource = ((extension_ftm_for_ic_t *)(configPtr->extension))->ftmClockSource;
    info.BDMMode = FTM_BDM_MODE_11;
    info.isTofIsrEnabled = false;
    info.enableInitializationTrigger = false;

    /* Initializes the FTM driver */
    status = FTM_DRV_Init(instance,
                          &info,
                          ftmState);
    DEV_ASSERT(STATUS_SUCCESS == status);

    param.nNumChannels = configPtr->nNumChannels;
    param.nMaxCountValue = MAX_COUNTER_VALUE;

    /* Configure a list of channels which are used */
    for (index = 0U; index < configPtr->nNumChannels; index++)
    {
        /* Get the hardware channel ID */
        channelConfig[index].hwChannelId = configPtr->inputChConfig[index].hwChannelId;
        channel = channelConfig[index].hwChannelId;
        /* Check the input capture operation mode */
        if ((configPtr->inputChConfig[index].inputCaptureMode == IC_TIMESTAMP_RISING_EDGE) || \
            (configPtr->inputChConfig[index].inputCaptureMode == IC_TIMESTAMP_FALLING_EDGE) || \
            (configPtr->inputChConfig[index].inputCaptureMode == IC_TIMESTAMP_BOTH_EDGES))
        {
            channelConfig[index].inputMode = FTM_EDGE_DETECT;
            channelConfig[index].measurementType = FTM_NO_MEASUREMENT;
            /* Check the type of signal detection */
            if (configPtr->inputChConfig[index].inputCaptureMode == IC_TIMESTAMP_RISING_EDGE)
            {
                channelConfig[index].edgeAlignement = FTM_RISING_EDGE;
            }
            else if (configPtr->inputChConfig[index].inputCaptureMode == IC_TIMESTAMP_FALLING_EDGE)
            {
                channelConfig[index].edgeAlignement = FTM_FALLING_EDGE;
            }
            else
            {
                channelConfig[index].edgeAlignement = FTM_BOTH_EDGES;
            }
        }
        else
        {
            /* Check the channel ID need to even number in the measurement mode */
            DEV_ASSERT((channel % 2U) == 0U);
            channelConfig[index].inputMode = FTM_SIGNAL_MEASUREMENT;
            channelConfig[index].edgeAlignement = FTM_NO_PIN_CONTROL;
            /* Check the type of measurement */
            if (configPtr->inputChConfig[index].inputCaptureMode == IC_MEASURE_RISING_EDGE_PERIOD)
            {
                channelConfig[index].measurementType = FTM_RISING_EDGE_PERIOD_MEASUREMENT;
            }
            else if (configPtr->inputChConfig[index].inputCaptureMode == IC_MEASURE_FALLING_EDGE_PERIOD)
            {
                channelConfig[index].measurementType = FTM_FALLING_EDGE_PERIOD_MEASUREMENT;
            }
            else if (configPtr->inputChConfig[index].inputCaptureMode == IC_MEASURE_PULSE_HIGH)
            {
                channelConfig[index].measurementType = FTM_PERIOD_ON_MEASUREMENT;
            }
            else
            {
                channelConfig[index].measurementType = FTM_PERIOD_OFF_MEASUREMENT;
            }
        }

        /* Set channels configuration from user */
        channelConfig[index].filterEn = configPtr->inputChConfig[index].filterEn;
        channelConfig[index].filterValue = configPtr->inputChConfig[index].filterValue;
        channelConfig[index].continuousModeEn = ((channel_extension_ftm_for_ic_t *)(configPtr->inputChConfig[index].channelExtension))->continuousModeEn;
        channelConfig[index].channelsCallbacksParams = configPtr->inputChConfig[index].channelCallbackParams;
        channelConfig[index].channelsCallbacks = configPtr->inputChConfig[index].channelCallbacks;

        /* Store some needed information into state structure */
        icState->channelConfigArray[index] = configPtr->inputChConfig[index].hwChannelId;
        icState->icChannelType[channel] = configPtr->inputChConfig[index].inputCaptureMode;
        icState->enableContinuousMode[channel] = channelConfig[index].continuousModeEn;
        icState->enableNotification[channel] = true;
        icState->channelCallbackParams[channel] = channelConfig[index].channelsCallbacksParams;
        icState->channelCallbacks[channel] = channelConfig[index].channelsCallbacks;
    }

    param.inputChConfig = channelConfig;
    /* Configure channels in input capture mode */
    status = FTM_DRV_InitInputCapture(instance,
                                      &param);

    return status;
}
#endif

#if (defined(IC_PAL_OVER_EMIOS))
/*FUNCTION**********************************************************************
 *
 * Function Name : IC_EmiosInit
 * Description   : This function initializes IC_PAL over EMIOS.
 *
 *END**************************************************************************/
static status_t IC_EmiosInit(uint32_t instance,
                             const ic_config_t * configPtr)
{
    ic_pal_state_t * icState;
    status_t status = STATUS_SUCCESS;
    uint8_t index = 0U;
    const IRQn_Type emiosIrq[eMIOS_INSTANCE_COUNT][(eMIOS_UC_COUNT / 2U)] = eMIOS_IRQS;

    /* Allocate one of the IC state structure for this instance */
    index = FindIcState(instance);
    icState = &g_icPalStatePtr[index];

    emios_common_param_t commonParam;
    uint8_t channel = 0U;
    uint8_t optionMode = 0U;
    emios_bus_select_t timeBase = EMIOS_BUS_SEL_A;
    emios_mc_mode_param_t mcParam;
    emios_input_capture_param_t icParam;
    emios_gpio_mode_param_t gpioParam;

    /* Get common parameters from user */
    commonParam.allowDebugMode = false;
    commonParam.lowPowerMode = false;
    commonParam.clkDivVal = ((extension_emios_for_ic_t *)(configPtr->extension))->clkDivVal;
    commonParam.enableGlobalPrescaler = ((extension_emios_for_ic_t *)(configPtr->extension))->enableGlobalPrescaler;
    commonParam.enableGlobalTimeBase = ((extension_emios_for_ic_t *)(configPtr->extension))->enableGlobalTimeBase;

    /* Initialize the global for a eMIOS group */
    EMIOS_DRV_InitGlobal((uint8_t)instance,
                          &commonParam);

    /* Get the information from user configuration */
    mcParam.mode = EMIOS_MODE_MCB_UP_COUNTER_INT_CLK;
    mcParam.period = MAX_COUNTER_VALUE;
    mcParam.filterInput = EMIOS_INPUT_FILTER_BYPASS;
    mcParam.filterEn = false;
    mcParam.triggerMode = EMIOS_TRIGGER_EDGE_ANY;

    /* Configure a list of channels which are used */
    for (index = 0U; index < configPtr->nNumChannels; index++)
    {
        mcParam.internalPrescaler = (emios_clock_internal_ps_t)((channel_extension_emios_for_ic_t *)(configPtr->inputChConfig[index].channelExtension))->prescaler;
        mcParam.internalPrescalerEn = true;

        switch (((channel_extension_emios_for_ic_t *)(configPtr->inputChConfig[index].channelExtension))->timebase)
        {
            case IC_BUS_SEL_A:
                /* Set channel to use as a time base */
                channel = 23U;
                timeBase = EMIOS_BUS_SEL_A;
                break;
            case IC_BUS_SEL_B:
                /* Set channel to use as a time base */
                channel = 0U;
                timeBase = EMIOS_BUS_SEL_BCDE;
                break;
            case IC_BUS_SEL_C:
                /* Set channel to use as a time base */
                channel = 8U;
                timeBase = EMIOS_BUS_SEL_BCDE;
                break;
            case IC_BUS_SEL_D:
                /* Set channel to use as a time base */
                channel = 16U;
                timeBase = EMIOS_BUS_SEL_BCDE;
                break;
            case IC_BUS_SEL_E:
                /* Set channel to use as a time base */
                channel = 24U;
                timeBase = EMIOS_BUS_SEL_BCDE;
                break;
            case IC_BUS_SEL_F:
                /* Set channel to use as a time base */
                channel = 22U;
                timeBase = EMIOS_BUS_SEL_F;
                break;
            case IC_BUS_SEL_INTERNAL:
                /* Set channel to use as a time base */
                channel = configPtr->inputChConfig[index].hwChannelId;
                timeBase = EMIOS_BUS_SEL_INTERNAL;
                break;
            default:
                /* Do nothing */
                break;
        }

        /* Initialize the counter mode */
        status = EMIOS_DRV_MC_InitCounterMode((uint8_t)instance,
                                              channel,
                                              &mcParam);

        channel = configPtr->inputChConfig[index].hwChannelId;
        icParam.mode = EMIOS_MODE_IC;
        icParam.timebase = timeBase;
        icParam.filterInput = (emios_input_filter_t)(configPtr->inputChConfig[index].filterValue);
        icParam.filterEn = configPtr->inputChConfig[index].filterEn;

        if (configPtr->inputChConfig[index].inputCaptureMode == IC_TIMESTAMP_RISING_EDGE)
        {
            icParam.inputCaptureMode = EMIOS_CAPTURE_TRIGGER_EDGE_RISING;
        }
        else if (configPtr->inputChConfig[index].inputCaptureMode == IC_TIMESTAMP_FALLING_EDGE)
        {
            icParam.inputCaptureMode = EMIOS_CAPTURE_TRIGGER_EDGE_FALLING;
        }
        else if (configPtr->inputChConfig[index].inputCaptureMode == IC_DISABLE_OPERATION)
        {
            /* Set default configure for an input pin */
            gpioParam.mode = EMIOS_MODE_GPIO_INPUT;
            gpioParam.filterEn = false;
            gpioParam.filterInput = EMIOS_INPUT_FILTER_BYPASS;
            gpioParam.triggerMode = EMIOS_TRIGGER_EDGE_ANY;

            /* Disable operation on the channel input */
            EMIOS_DRV_InitGpioMode((uint8_t)instance,
                                   channel,
                                   &gpioParam);
        }
        else
        {
            optionMode = (uint8_t)configPtr->inputChConfig[index].inputCaptureMode - 1U;
            icParam.inputCaptureMode = (emios_input_capture_mode_t)optionMode;
        }

        if (configPtr->inputChConfig[index].inputCaptureMode != IC_DISABLE_OPERATION)
        {
            /* Initialize the input capture mode for each channel */
            status = EMIOS_DRV_IC_InitInputCaptureMode((uint8_t)instance,
                                                       channel,
                                                       &icParam);
        }

        EMIOS_DRV_ChannelEnableInterruptRequest((uint8_t)instance, channel);
        /* Enable EMIOS interrupt */
        INT_SYS_EnableIRQ(emiosIrq[instance][(channel>>1U)]);

        /* Store some needed information into state structure */
        icState->channelConfigArray[index] = channel;
        icState->icChannelType[channel] = configPtr->inputChConfig[index].inputCaptureMode;
        icState->timeBaseSelection[channel] = icParam.timebase;
        icState->filterEn[channel] = icParam.filterEn;
        icState->filterInput[channel] = icParam.filterInput;
        icState->enableNotification[channel] = true;
        icState->channelCallbackParams[channel] = configPtr->inputChConfig[index].channelCallbackParams;
        icState->channelCallbacks[channel] = configPtr->inputChConfig[index].channelCallbacks;
    }

    /* Enable the global eMIOS to start counter */
    EMIOS_DRV_EnableGlobalEmios((uint8_t)instance);

    return status;
}
#endif

#if (defined(IC_PAL_OVER_ETIMER))
/*FUNCTION**********************************************************************
 *
 * Function Name : IC_EtimerInit
 * Description   : This function initializes the input capture mode over ETIMER.
 *
 *END**************************************************************************/
static status_t IC_EtimerInit(uint32_t instance,
                              const ic_config_t * configPtr)
{
    status_t status = STATUS_SUCCESS;
    uint8_t index = 0U;
    uint8_t i = 0U;
    uint16_t channel = 0U;
    uint16_t maskChannel = 0U;
    etimer_cptmode_t capCtr[ETIMER_NUM_COMPARE_REGISTERS];
    ic_pal_state_t * icState;
    static const IRQn_Type etimerIrqId[ETIMER_INSTANCE_COUNT][ETIMER_CH_MAX_IRQS] = ETIMER_IRQS;

    /* Allocate one of the IC state structure for this instance */
    index = FindIcState(instance);
    icState = &g_icPalStatePtr[index];

    etimer_user_channel_config_t userChannelConfig;

    /* Initialize the ETIMER module */
    ETIMER_DRV_Init((uint16_t)instance);

    userChannelConfig.coreSettings = ETIMER_COUNTER_NORMAL;
    userChannelConfig.outputPin.enable = false;
    userChannelConfig.outputPin.polarity = ETIMER_POLARITY_POSITIVE;
    userChannelConfig.countDirection = ETIMER_COUNT_UP;
    userChannelConfig.compareOutputControl = ETIMER_OUTMODE_SOFTWARE;
    userChannelConfig.compareMode = ETIMER_CMPMODE_COMP1_UP_COMP2_UP;
    userChannelConfig.primaryInput.polarity = ETIMER_POLARITY_POSITIVE;
    userChannelConfig.secondaryInput.polarity = ETIMER_POLARITY_POSITIVE;

    for (index = 0U; index < ETIMER_NUM_COMPARE_REGISTERS; index++)
    {
        userChannelConfig.compareValues[index] = 0U;
        userChannelConfig.compareLoading[index] = ETIMER_CLC_NEVER;
        capCtr[index] = ETIMER_CPTMODE_DISABLED;
    }

    /* Configure a list of channels which are used */
    for (index = 0U; index < configPtr->nNumChannels; index++)
    {
        channel = configPtr->inputChConfig[index].hwChannelId;
        maskChannel |= (uint16_t)(1U << channel);

        if (((channel_extension_etimer_for_ic_t *)configPtr->inputChConfig[index].channelExtension)->continuousModeEn == true)
        {
            userChannelConfig.timerMode = ETIMER_MODE_INPUT_CAPTURE;
        }
        else
        {
            userChannelConfig.timerMode = ETIMER_MODE_INPUT_CAPTURE_ONESHOT;
        }

        if (configPtr->inputChConfig[index].filterEn)
        {
            userChannelConfig.inputFilter.samples = (etimer_filt_cnt_t)configPtr->inputChConfig[index].filterValue;
            userChannelConfig.inputFilter.rate = ((channel_extension_etimer_for_ic_t *)configPtr->inputChConfig[index].channelExtension)->rate;
        }

        userChannelConfig.primaryInput.source = (etimer_input_source_t)(((channel_extension_etimer_for_ic_t *)configPtr->inputChConfig[index].channelExtension)->prescaler);;
        userChannelConfig.secondaryInput.source = (etimer_input_source_t)channel;
        userChannelConfig.captureWords = 1U;
        userChannelConfig.interruptEnableMask = ETIMER_CH_IRQ_SOURCE_ICF1IE;

        /* Check the type of measurement */
        if (configPtr->inputChConfig[index].inputCaptureMode == IC_TIMESTAMP_RISING_EDGE)
        {
            capCtr[0] = ETIMER_CPTMODE_RISING;
        }
        else if (configPtr->inputChConfig[index].inputCaptureMode == IC_TIMESTAMP_FALLING_EDGE)
        {
            capCtr[0] = ETIMER_CPTMODE_FALLING;
        }
        else if (configPtr->inputChConfig[index].inputCaptureMode == IC_TIMESTAMP_BOTH_EDGES)
        {
            capCtr[0] = ETIMER_CPTMODE_BOTH;
        }
        else if (configPtr->inputChConfig[index].inputCaptureMode == IC_MEASURE_RISING_EDGE_PERIOD)
        {
            capCtr[0] = ETIMER_CPTMODE_RISING;
            userChannelConfig.captureWords = 2U;
        }
        else if (configPtr->inputChConfig[index].inputCaptureMode == IC_MEASURE_FALLING_EDGE_PERIOD)
        {
            capCtr[0] = ETIMER_CPTMODE_FALLING;
            userChannelConfig.captureWords = 2U;
        }
        else if (configPtr->inputChConfig[index].inputCaptureMode == IC_MEASURE_PULSE_HIGH)
        {
            capCtr[0] = ETIMER_CPTMODE_RISING;
            capCtr[1] = ETIMER_CPTMODE_FALLING;
            userChannelConfig.interruptEnableMask = ETIMER_CH_IRQ_SOURCE_ICF2IE;
        }
        else if (configPtr->inputChConfig[index].inputCaptureMode == IC_MEASURE_PULSE_LOW)
        {
            capCtr[0] = ETIMER_CPTMODE_FALLING;
            capCtr[1] = ETIMER_CPTMODE_RISING;
            userChannelConfig.interruptEnableMask = ETIMER_CH_IRQ_SOURCE_ICF2IE;
        }
        else
        {
            /* Nothing to do */
        }

        for (i = 0U; i < ETIMER_NUM_COMPARE_REGISTERS; i++)
        {
            userChannelConfig.captureControl[i] = capCtr[i];
        }

        /* Initialize the channel at the input capture mode */
        ETIMER_DRV_InitChannel((uint16_t)instance, channel, &userChannelConfig);
        /* Enable the input capture */
        ETIMER_DRV_StartCapture((uint16_t)instance, channel);
        /* Enable interrupt on channel */
        INT_SYS_EnableIRQ(etimerIrqId[instance][channel]);

        /* Store some needed information into state structure */
        icState->channelConfigArray[index] = channel;
        icState->icChannelType[channel] = configPtr->inputChConfig[index].inputCaptureMode;
        icState->enableNotification[channel] = true;
        icState->channelCallbackParams[channel] = configPtr->inputChConfig[index].channelCallbackParams;
        icState->channelCallbacks[channel] = configPtr->inputChConfig[index].channelCallbacks;
    }

    /* Start the channel counting */
    ETIMER_DRV_StartTimerChannels((uint16_t)instance, maskChannel);

    return status;
}

/*FUNCTION**********************************************************************
 *
 * Function Name : IC_Etimer_SetChannelMode
 * Description   : This function will change the mode operation in the input capture.
 *
 *END**************************************************************************/
static void IC_Etimer_SetChannelMode(uint32_t instance,
                                     uint8_t channel,
                                     ic_option_mode_t channelMode)
{
    DEV_ASSERT(instance < ETIMER_INSTANCE_COUNT);
    DEV_ASSERT(channel < ETIMER_CH_COUNT);
    /* Table of base addresses for ETIMER instances */
    static ETIMER_Type * const s_etimerBase[ETIMER_INSTANCE_COUNT] = ETIMER_BASE_PTRS;
    ETIMER_Type * base;
    base = s_etimerBase[instance];
    uint8_t captureWords = 1U;
    etimer_cptmode_t captureControl[2] = { ETIMER_CPTMODE_DISABLED, ETIMER_CPTMODE_DISABLED };
    base->CH[channel].INTDMA &= ~(ETIMER_CH_IRQ_FLAGS_ICF1 | ETIMER_CH_IRQ_FLAGS_ICF2);

    switch (channelMode)
    {
        case IC_TIMESTAMP_RISING_EDGE:
            captureControl[0] = ETIMER_CPTMODE_RISING;
            base->CH[channel].INTDMA |= ETIMER_CH_IRQ_FLAGS_ICF1;
            break;
        case IC_TIMESTAMP_FALLING_EDGE:
            captureControl[0] = ETIMER_CPTMODE_FALLING;
            base->CH[channel].INTDMA |= ETIMER_CH_IRQ_FLAGS_ICF1;
            break;
        case IC_TIMESTAMP_BOTH_EDGES:
            captureControl[0] = ETIMER_CPTMODE_BOTH;
            base->CH[channel].INTDMA |= ETIMER_CH_IRQ_FLAGS_ICF1;
            break;
        case IC_MEASURE_RISING_EDGE_PERIOD:
            captureControl[0] = ETIMER_CPTMODE_RISING;
            base->CH[channel].INTDMA |= ETIMER_CH_IRQ_FLAGS_ICF1;
            captureWords = 2U;
            break;
        case IC_MEASURE_FALLING_EDGE_PERIOD:
            captureControl[0] = ETIMER_CPTMODE_FALLING;
            base->CH[channel].INTDMA |= ETIMER_CH_IRQ_FLAGS_ICF1;
            captureWords = 2U;
            break;
        case IC_MEASURE_PULSE_HIGH:
            captureControl[0] = ETIMER_CPTMODE_RISING;
            captureControl[1] = ETIMER_CPTMODE_FALLING;
            base->CH[channel].INTDMA |= ETIMER_CH_IRQ_FLAGS_ICF2;
            break;
        case IC_MEASURE_PULSE_LOW:
            captureControl[0] = ETIMER_CPTMODE_FALLING;
            captureControl[1] = ETIMER_CPTMODE_RISING;
            base->CH[channel].INTDMA |= ETIMER_CH_IRQ_FLAGS_ICF2;
            break;
        default:
            /* nothing to do */
            break;
    }

    /* Capture counter value using external inputs */
    base->CH[channel].CCCTRL &= ((~ETIMER_CCCTRL_CFWM_MASK) & (~ETIMER_CCCTRL_CPT1MODE_MASK) & (~ETIMER_CCCTRL_CPT2MODE_MASK));
    base->CH[channel].CCCTRL |= ETIMER_CCCTRL_CFWM(captureWords) | ETIMER_CCCTRL_CPT1MODE(captureControl[0]) | ETIMER_CCCTRL_CPT2MODE(captureControl[1]);
}
#endif

/*FUNCTION**********************************************************************
 *
 * Function Name : IC_Init
 * Description   : This function will initialize the IC PAL instance, including the
 * other platform specific HW units used together in the input capture mode. This
 * function configures a group of channels in instance to detect or measure the
 * input signal.
 *
 * Implements    : IC_Init_Activity
 *END**************************************************************************/
status_t IC_Init(const ic_instance_t * const instance,
                 const ic_config_t * configPtr)
{
    DEV_ASSERT(configPtr != NULL);
    DEV_ASSERT(instance != NULL);
    DEV_ASSERT(instance->instIdx < IC_PAL_INSTANCE_MAX);
    ic_pal_state_t * icState;
    status_t status = STATUS_SUCCESS;
    uint8_t index = 0U;

    /* Allocate one of the IC state structure for this instance */
    index = icAllocateState(icStateIsAllocated, icStateInstanceMapping, instance->instIdx);
    icState = &g_icPalStatePtr[index];
    DEV_ASSERT(icState->nNumChannels == 0x0U);

    /* Initialize the internal context to default value */
    icState->nNumChannels = configPtr->nNumChannels;
    for (index = 0U; index < IC_PAL_NUM_OF_CHANNEL_MAX; index++)
    {
        icState->channelConfigArray[index] = 0U;
        icState->icChannelType[index] = IC_DISABLE_OPERATION;
        icState->enableNotification[index] = false;
        icState->channelCallbackParams[index] = NULL;
        icState->channelCallbacks[index] = NULL;
        #if (defined(IC_PAL_OVER_FTM))
        icState->enableContinuousMode[index] = false;
        #endif
        #if (defined(IC_PAL_OVER_ETIMER))
        icState->measurementResults[index] = 0U;
        #endif
    }

#if (defined(IC_PAL_OVER_FTM))
    DEV_ASSERT(instance->instType == IC_INST_TYPE_FTM);
    status = IC_FtmInit(instance->instIdx, configPtr);
#endif

#if (defined(IC_PAL_OVER_EMIOS))
    DEV_ASSERT(instance->instType == IC_INST_TYPE_EMIOS);
    status = IC_EmiosInit(instance->instIdx, configPtr);
#endif

#if (defined(IC_PAL_OVER_ETIMER))
    DEV_ASSERT(instance->instType == IC_INST_TYPE_ETIMER);
    status = IC_EtimerInit(instance->instIdx, configPtr);
#endif

    return status;
}

/*FUNCTION**********************************************************************
 *
 * Function Name : IC_Deinit
 * Description   : This function will disable the input capture mode. The driver
 * can't be used again until reinitialized. The context structure is no longer
 * needed by the driver and can be freed after calling this function.
 *
 * Implements    : IC_Deinit_Activity
 *END**************************************************************************/
status_t IC_Deinit(const ic_instance_t * const instance)
{
    DEV_ASSERT(instance != NULL);
    DEV_ASSERT(instance->instIdx < IC_PAL_INSTANCE_MAX);
    uint8_t index = 0U;
    ic_pal_state_t * icState;

    /* Allocate one of the IC state structure for this instance */
    index = FindIcState(instance->instIdx);
    icState = &g_icPalStatePtr[index];

#if (defined(IC_PAL_OVER_FTM))
    DEV_ASSERT(instance->instType == IC_INST_TYPE_FTM);
    /* Disable the input capture over FTM */
    (void)FTM_DRV_Deinit(instance->instIdx);
    ftm_state_t * ftmState = &g_ftmState[index];
#endif

#if (defined(IC_PAL_OVER_EMIOS))
    DEV_ASSERT(instance->instType == IC_INST_TYPE_EMIOS);
    uint8_t channel;

    for (index = 0U; index < icState->nNumChannels; index++)
    {
        channel = icState->channelConfigArray[index];
        /* Disable channels in the input capture over EMIOS */
        EMIOS_DRV_DeInitChannel((uint8_t)instance->instIdx,
                                channel);
    }

    /* Disable the global of EMIOS */
    EMIOS_DRV_DisableGlobalEmios((uint8_t)instance->instIdx);
#endif

#if (defined(IC_PAL_OVER_ETIMER))
    DEV_ASSERT(instance->instType == IC_INST_TYPE_ETIMER);
    /* Disable the input capture over ETIMER */
    ETIMER_DRV_Deinit((uint16_t)instance->instIdx);
#endif

    /* De-Initialize the internal context to default value */
    icState->nNumChannels = 0U;
    for (index = 0U; index < IC_PAL_NUM_OF_CHANNEL_MAX; index++)
    {
        icState->channelConfigArray[index] = 0U;
        icState->icChannelType[index] = IC_DISABLE_OPERATION;
        icState->enableNotification[index] = false;
        #if (defined(IC_PAL_OVER_FTM))
        icState->enableContinuousMode[index] = false;
        ftmState->enableNotification[index] = false;
        #endif
        #if (defined(IC_PAL_OVER_ETIMER))
        icState->measurementResults[index] = 0U;
        #endif
    }

    /* De-Allocate the instance which is not used */
    icFreeState(icStateIsAllocated, icStateInstanceMapping, instance->instIdx);

    return STATUS_SUCCESS;
}

/*FUNCTION**********************************************************************
 *
 * Function Name : IC_StartChannel
 * Description   : This function start channel counting.
 *
 * Implements    : IC_StartChannel_Activity
 *END**************************************************************************/
void IC_StartChannel(const ic_instance_t * const instance,
                     uint8_t channel)
{
    DEV_ASSERT(instance != NULL);
    DEV_ASSERT(instance->instIdx < IC_PAL_INSTANCE_MAX);
    DEV_ASSERT(channel < IC_PAL_NUM_OF_CHANNEL_MAX);

#if (defined(IC_PAL_OVER_FTM))
    uint8_t index = 0U;
    const ic_pal_state_t * icState;
    status_t status = STATUS_SUCCESS;

    /* Allocate one of the IC state structure for this instance */
    index = FindIcState(instance->instIdx);
    icState = &g_icPalStatePtr[index];

    /* Re-start the channel mode */
    status = IC_SetChannelMode(instance,
                               channel,
                               icState->icChannelType[channel]);
    (void)status;
#endif

#if (defined(IC_PAL_OVER_EMIOS))
    /* Enable the channel clock */
    EMIOS_DRV_ChannelEnableClk((uint8_t)instance->instIdx,
                               channel);
#endif

#if (defined(IC_PAL_OVER_ETIMER))
    ETIMER_DRV_StartCapture((uint16_t)instance->instIdx,
                            (uint16_t)channel);
    ETIMER_DRV_StartTimerChannels((uint16_t)instance->instIdx,
                                  (uint16_t)(1U << channel));
#endif
}

/*FUNCTION**********************************************************************
 *
 * Function Name : IC_StopChannel
 * Description   : This function will stop channel counting.
 *
 * Implements    : IC_StopChannel_Activity
 *END**************************************************************************/
void IC_StopChannel(const ic_instance_t * const instance,
                    uint8_t channel)
{
    DEV_ASSERT(instance != NULL);
    DEV_ASSERT(instance->instIdx < IC_PAL_INSTANCE_MAX);
    DEV_ASSERT(channel < IC_PAL_NUM_OF_CHANNEL_MAX);

#if (defined(IC_PAL_OVER_FTM))
    /* Disable pin not used for FTM */
    (void)FTM_DRV_SetOutputlevel(instance->instIdx,
                                 channel,
                                 0x0U);
#endif

#if (defined(IC_PAL_OVER_EMIOS))
    /* Disable individual channel by stopping its respective clock*/
    EMIOS_DRV_ChannelDisableClk((uint8_t)instance->instIdx,
                                channel);
#endif

#if (defined(IC_PAL_OVER_ETIMER))
    ETIMER_DRV_StopCapture((uint16_t)instance->instIdx,
                           (uint16_t)channel);
    ETIMER_DRV_StopTimerChannels((uint16_t)instance->instIdx,
                                 (uint16_t)(1U << channel));
#endif
}

/*FUNCTION**********************************************************************
 *
 * Function Name : IC_SetChannelMode
 * Description   : This function is used to change the channel mode at run time or
 * when stopping channel. The channel mode is selected in the ic_option_mode_t
 * enumeration type.
 *
 * Implements    : IC_SetChannelMode_Activity
 *END**************************************************************************/
status_t IC_SetChannelMode(const ic_instance_t * const instance,
                           uint8_t channel,
                           ic_option_mode_t channelMode)
{
    DEV_ASSERT(instance != NULL);
    DEV_ASSERT(instance->instIdx < IC_PAL_INSTANCE_MAX);
    DEV_ASSERT(channel < IC_PAL_NUM_OF_CHANNEL_MAX);
    uint8_t index = 0U;
    ic_pal_state_t * icState;
    status_t status = STATUS_SUCCESS;

    /* Allocate one of the IC state structure for this instance */
    index = FindIcState(instance->instIdx);
    icState = &g_icPalStatePtr[index];

#if (defined(IC_PAL_OVER_FTM))
    bool contModeEnable;
    index = (uint8_t)channelMode;
    ftm_ic_op_mode_t inputMode = (ftm_ic_op_mode_t)index;

    if (true == icState->enableContinuousMode[channel])
    {
        contModeEnable = true;
    }
    else
    {
        contModeEnable = false;
    }

    /* Set operation mode for channel input */
    status = FTM_IC_DRV_SetChannelMode(instance->instIdx,
                                       channel,
                                       inputMode,
                                        contModeEnable);
#endif

#if (defined(IC_PAL_OVER_EMIOS))
    emios_input_capture_param_t icParam;
    emios_gpio_mode_param_t gpioParam;
    uint8_t optionMode = 0U;

    if (channelMode == IC_DISABLE_OPERATION)
    {
        /* Set default configure for an input pin */
        gpioParam.mode = EMIOS_MODE_GPIO_INPUT;
        gpioParam.filterEn = false;
        gpioParam.filterInput = EMIOS_INPUT_FILTER_BYPASS;
        gpioParam.triggerMode = EMIOS_TRIGGER_EDGE_ANY;

        /* Disable operation on the channel input */
        EMIOS_DRV_InitGpioMode((uint8_t)instance->instIdx,
                               channel,
                               &gpioParam);
    }
    else
    {
        icParam.mode = EMIOS_MODE_IC;
        icParam.timebase = icState->timeBaseSelection[channel];
        icParam.filterInput = icState->filterInput[channel];
        icParam.filterEn = icState->filterEn[channel];

        if (channelMode == IC_TIMESTAMP_RISING_EDGE)
        {
            icParam.inputCaptureMode = EMIOS_CAPTURE_TRIGGER_EDGE_RISING;
        }
        else if (channelMode == IC_TIMESTAMP_FALLING_EDGE)
        {
            icParam.inputCaptureMode = EMIOS_CAPTURE_TRIGGER_EDGE_FALLING;
        }
        else
        {
            optionMode = (uint8_t)(channelMode) - 1U;
            icParam.inputCaptureMode = (emios_input_capture_mode_t)optionMode;
        }

        /* Initialize the input capture mode for each channel */
        status = EMIOS_DRV_IC_InitInputCaptureMode((uint8_t)instance->instIdx,
                                                   channel,
                                                   &icParam);
    }
#endif

#if (defined(IC_PAL_OVER_ETIMER))
    IC_Etimer_SetChannelMode(instance->instIdx,
                             channel,
                             channelMode);
#endif

    /* Update the channel mode */
    icState->icChannelType[channel] = channelMode;

    return status;
}

/*FUNCTION**********************************************************************
 *
 * Function Name : IC_GetMeasurement
 * Description   : This function will get the value of measured signal in ticks.
 *
 * Implements    : IC_GetMeasurement_Activity
 *END**************************************************************************/
uint16_t IC_GetMeasurement(const ic_instance_t * const instance,
                           uint8_t channel)
{
    DEV_ASSERT(instance != NULL);
    DEV_ASSERT(instance->instIdx < IC_PAL_INSTANCE_MAX);
    DEV_ASSERT(channel < IC_PAL_NUM_OF_CHANNEL_MAX);
    uint16_t value = 0U;

#if (defined(IC_PAL_OVER_FTM))
    /* Get the measured value over the FTM */
    value = FTM_DRV_GetInputCaptureMeasurement(instance->instIdx,
                                               channel);
#endif

#if (defined(IC_PAL_OVER_EMIOS))
    status_t status = STATUS_SUCCESS;
    uint32_t retValue = 0U;

    /* Get the measured value over the EMIOS */
    status = EMIOS_DRV_IC_GetLastMeasurement((uint8_t)instance->instIdx,
                                             channel,
                                             &retValue);
    DEV_ASSERT(STATUS_SUCCESS == status);
    (void) status;
    value = (uint16_t)retValue;
#endif

#if (defined(IC_PAL_OVER_ETIMER))
    uint8_t index = 0U;
    const ic_pal_state_t * icState;

    /* Allocate one of the IC state structure for this instance */
    index = FindIcState(instance->instIdx);
    icState = &g_icPalStatePtr[index];

    value = icState->measurementResults[channel];
#endif

    return value;
}

/*FUNCTION**********************************************************************
 *
 * Function Name : IC_EnableNotification
 * Description   : This function enables channel notification.
 *
 * Implements    : IC_EnableNotification_Activity
 *END**************************************************************************/
void IC_EnableNotification(const ic_instance_t * const instance,
                           uint8_t channel)
{
    DEV_ASSERT(instance != NULL);
    DEV_ASSERT(instance->instIdx < IC_PAL_INSTANCE_MAX);
    DEV_ASSERT(channel < IC_PAL_NUM_OF_CHANNEL_MAX);
    uint8_t index = 0U;
    ic_pal_state_t * icState;

    /* Allocate one of the IC state structure for this instance */
    index = FindIcState(instance->instIdx);
    icState = &g_icPalStatePtr[index];

    /* Enable notification */
    icState->enableNotification[channel] = true;

#if (defined(IC_PAL_OVER_FTM))
    ftm_state_t * ftmState = ftmStatePtr[instance->instIdx];
    ftmState->enableNotification[channel] = true;
#endif
}

/*FUNCTION**********************************************************************
 *
 * Function Name : IC_DisableNotification
 * Description   : This function disables channel notification.
 *
 * Implements    : IC_DisableNotification_Activity
 *END**************************************************************************/
void IC_DisableNotification(const ic_instance_t * const instance,
                            uint8_t channel)
{
    DEV_ASSERT(instance != NULL);
    DEV_ASSERT(instance->instIdx < IC_PAL_INSTANCE_MAX);
    DEV_ASSERT(channel < IC_PAL_NUM_OF_CHANNEL_MAX);
    uint8_t index = 0U;
    ic_pal_state_t * icState;

    /* Allocate one of the IC state structure for this instance */
    index = FindIcState(instance->instIdx);
    icState = &g_icPalStatePtr[index];

    /* Disable notification */
    icState->enableNotification[channel] = false;

#if (defined(IC_PAL_OVER_FTM))
    ftm_state_t * ftmState = ftmStatePtr[instance->instIdx];
    ftmState->enableNotification[channel] = false;
#endif
}

/*FUNCTION**********************************************************************
 *
 * Function Name : IC_IrqHandler
 * Description   : Interrupt handler for IC PAL over EMIOS.
 * This is not a public API as it is called by IRQ whenever an interrupt occurs.
 *
 *END**************************************************************************/
void IC_IrqHandler(uint32_t instance, uint8_t channel)
{
    DEV_ASSERT(instance < IC_PAL_INSTANCE_MAX);
    DEV_ASSERT(channel < IC_PAL_NUM_OF_CHANNEL_MAX);
    uint8_t index = 0U;
#if (defined (IC_PAL_OVER_ETIMER))
    ic_pal_state_t * icState;
#else
    const ic_pal_state_t * icState;
#endif

    /* Allocate one of the IC state structure for this instance */
    index = FindIcState(instance);
    icState = &g_icPalStatePtr[index];

#if (defined (IC_PAL_OVER_ETIMER))
    uint16_t value = 0U;
    uint16_t captureRegister = 0U;
    uint16_t captureResultBuffer1[2] = {0U, 0U};
    uint16_t captureResultBuffer2[2] = {0U, 0U};
    uint8_t captureWordsToRead = 1U;
    uint16_t returnCode = 0U;

    if ((icState->icChannelType[channel] == IC_MEASURE_RISING_EDGE_PERIOD) || (icState->icChannelType[channel] == IC_MEASURE_FALLING_EDGE_PERIOD))
    {
        captureWordsToRead = 2U;
    }

    returnCode = ETIMER_DRV_GetCaptureValue((uint16_t)instance,
                                            (uint16_t)channel,
                                            captureRegister,
                                            captureResultBuffer1,
                                            captureWordsToRead);

    if (returnCode == 0U)
    {
        value = 0U;
    }
    else if (returnCode == 1U)
    {
        value = captureResultBuffer1[0];
    }
    else
    {
        if (captureResultBuffer1[1] > captureResultBuffer1[0])
        {
            value = captureResultBuffer1[1] - captureResultBuffer1[0];
        }
        else
        {
            value = captureResultBuffer1[1] + (ETIMER_CH_MAX_TIMER_COUNT - captureResultBuffer1[0]);
        }
    }

    if ((icState->icChannelType[channel] == IC_MEASURE_PULSE_HIGH) || (icState->icChannelType[channel] == IC_MEASURE_PULSE_LOW))
    {
        captureRegister = 1U;
        returnCode = ETIMER_DRV_GetCaptureValue((uint16_t)instance,
                                                (uint16_t)channel,
                                                captureRegister,
                                                captureResultBuffer2,
                                                captureWordsToRead);
        if (captureResultBuffer2[0] > captureResultBuffer1[0])
        {
            value = captureResultBuffer2[0] - captureResultBuffer1[0];
        }
        else
        {
            value = captureResultBuffer2[0] + (ETIMER_CH_MAX_TIMER_COUNT - captureResultBuffer1[0]);
        }
    }

    if (icState->icChannelType[channel] == IC_DISABLE_OPERATION)
    {
        value = 0U;
    }

    (void)returnCode;

    /* Get the time stamp of the event */
    icState->measurementResults[channel] = value;
#endif

    if ((icState->channelCallbacks[channel] != NULL) && (icState->enableNotification[channel] == true))
    {
        /* Call to callback function */
        (icState->channelCallbacks[channel])(IC_EVENT_MEASUREMENT_COMPLETE, icState->channelCallbackParams[channel]);
    }

#if (defined (IC_PAL_OVER_EMIOS))
    /* Clear interrupt flag */
    EMIOS_DRV_ClrFlagState(instance, channel);
#endif

#if (defined (IC_PAL_OVER_ETIMER))
    /* Clear interrupt flag */
    ETIMER_DRV_ClearInterruptStatus((uint16_t)instance,
                                    (ETIMER_CH_IRQ_FLAGS_ICF1 | ETIMER_CH_IRQ_FLAGS_ICF2),
                                    channel);
#endif
}

/*******************************************************************************
 * EOF
 ******************************************************************************/
